package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.*;
import com.yunzhi.nanyang.entity.SysLogin;
import com.yunzhi.nanyang.service.ISysLoginService;
import com.yunzhi.nanyang.service.ISysUserDataScopeService;
import com.yunzhi.nanyang.service.ITaOrgService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ISysUserService;
import com.yunzhi.nanyang.entity.SysUser;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 用户表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-07
 */

@Api(tags = "用户表")
@RestController
@RequestMapping("/")
public class SysUserController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(SysUserController.class);

    @Autowired
    public ISysUserService iSysUserService;

    @Autowired
    public ISysLoginService iSysLoginService;

    @Autowired
    public ISysUserDataScopeService iSysUserDataScopeService;

    @Autowired
    public ITaOrgService iTaOrgService;

    @Value("${yz.default.password}")
    public String defaultPassword;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/user",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean sysUserList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                    @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                    @ApiParam("机构") @RequestParam(value ="org_id", required = false) String orgId,
                                    @ApiParam("名称") @RequestParam(value ="userName", required = false) String userName,
                                    @ApiParam("手机号") @RequestParam(value ="phone", required = false) String phone) throws Exception{

        SysUser sysUser = currentUser();
        String userId = sysUser.getUserId();
        boolean isAdmin = Constants.ADMIN_ID.equals(userId);
        if (!isAdmin) {
            // 不支持非管理员使用机构查询
            orgId = sysUser.getOrgId();
        }

        IPage<SysUser> pg = new Page<>(pageNum, pageSize);

        IPage<SysUser> result = iSysUserService.getPageBy(pg, userId, orgId, userName, phone);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param sysUser 实体对象
     * @return
     */
    @RequestMapping(value="/admin/user",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    @SaCheckPermission("user:add")
    public ResponseBean sysUserAdd(@ApiParam("保存内容") @RequestBody SysUser sysUser) throws Exception{
        String userId = currentUser().getUserId();
        sysUser.setUserId(null);
        sysUser.setCreateUser(userId);

        checkOrgAccess(sysUser.getOrgId());

        if (StringUtils.isEmpty(sysUser.getUserName()) || StringUtils.isEmpty(sysUser.getPhone())) {
            return ResponseBean.error("姓名或者手机不能为空");
        }

        if (StringUtils.isEmpty(sysUser.getOrgId())) {
            return ResponseBean.error("所属机构不能为空");
        }

        int cntPhone = iSysUserService.countBy("phone", sysUser.getPhone(), true);
        if (cntPhone > 0) {
            return ResponseBean.error("手机号码重复");
        }

        if (!StringUtils.isEmpty(sysUser.getLoginName())) {
            int cnt = iSysLoginService.countBy("login_name", sysUser.getLoginName(), true);
            if (cnt > 0) {
                return ResponseBean.error("登录名重复");
            }
        }

        if (iSysUserService.save(sysUser)){
            iTaOrgService.updateNum(sysUser.getOrgId(), "worker_num", 1);
            iSysUserDataScopeService.addNew(sysUser.getOrgId(), sysUser.getUserId());

            if (!StringUtils.isEmpty(sysUser.getLoginName())) {
                addNewSysLogin(sysUser);
            }

            return ResponseBean.success(sysUser);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/user/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    @SaCheckPermission("user:delete")
    public ResponseBean sysUserDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        SysUser sysUser = iSysUserService.getExistBy("user_id", id, false, true);
        if (sysUser == null) {
            return ResponseBean.error("用户不存在");
        }

        boolean canAccess = iSysUserService.canAccessData(sysUser.getOrgId(), currentUser().getUserId());
        if (!canAccess) {
            return ResponseBean.error("无机构操作权限");
        }

        if(iSysUserService.removeLogicById(id)){
            iTaOrgService.updateNum(sysUser.getOrgId(), "worker_num", -1);
            iSysLoginService.removeLogicByUser(id);

            // 解绑人员
            iTaPersonService.unBindUser(id);

            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param sysUser 实体对象
     * @return
     */
    @RequestMapping(value="/admin/user/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    @SaCheckPermission("user:update")
    public ResponseBean sysUserUpdate(@ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("更新内容") @RequestBody SysUser sysUser) throws Exception{
        String userId = currentUser().getUserId();
        sysUser.setUserId(id);
        // 不支持更新创建人
        sysUser.setCreateUser(null);

        checkOrgAccess(sysUser.getOrgId());

        if (StringUtils.isEmpty(sysUser.getUserName()) || StringUtils.isEmpty(sysUser.getPhone())) {
            return ResponseBean.error("姓名或者手机不能为空");
        }

        if (StringUtils.isEmpty(sysUser.getOrgId())) {
            return ResponseBean.error("所属机构不能为空");
        }

        SysUser origin = iSysUserService.getByButNot("phone", sysUser.getPhone(), "user_id", id,true);
        if (origin != null) {
            return ResponseBean.error("手机号码重复");
        }

        boolean canAccess = iSysUserService.canAccessData(sysUser.getOrgId(), userId);
        if (!canAccess) {
            return ResponseBean.error("无机构操作权限");
        }

        if (!StringUtils.isEmpty(sysUser.getLoginName())) {
            SysLogin sysLogin = iSysLoginService.getByButNot("login_name", sysUser.getLoginName(), "user_id", id, true);
            if (sysLogin != null) {
                return ResponseBean.error("登录名重复");
            }
        }

        if (iSysUserService.updateById(sysUser)){
            iSysUserDataScopeService.addNew(sysUser.getOrgId(), sysUser.getUserId());
            if (!StringUtils.isEmpty(sysUser.getLoginName())) {
                SysLogin sysLogin = iSysLoginService.getExistBy("user_id", id, false, true);
                if (sysLogin == null) {
                    addNewSysLogin(sysUser);
                } else if (!sysLogin.getLoginName().equals(sysUser.getLoginName())) {
                    sysLogin.setLoginName(sysUser.getLoginName());
                    iSysLoginService.updateById(sysLogin);
                }
            }

            return ResponseBean.success(iSysUserService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    private SysLogin addNewSysLogin(SysUser sysUser) {
        SysLogin sysLogin = new SysLogin();
        sysLogin.setUserId(sysUser.getUserId());
        sysLogin.setLoginName(sysUser.getLoginName());
        sysLogin.setPassword(EncryptUtils.md5(EncryptUtils.md5(defaultPassword), sysLogin.getUserId()));
        sysLogin.setStatus(Constants.STATUS_NORMAL);
        iSysLoginService.save(sysLogin);
        return sysLogin;
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/user/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    @SaCheckPermission("user:view")
    public ResponseBean sysUserGet(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        SysUser sysUser = iSysUserService.getExistBy("user_id", id, false, true);

        boolean canAccess = iSysUserService.canAccessData(sysUser.getOrgId(), currentUser().getUserId());
        if (!canAccess) {
            return ResponseBean.error("无权限操作");
        }

        SysLogin sysLogin = iSysLoginService.getExistBy("user_id", id, false, true);
        if (null != sysLogin) {
            sysUser.setLoginName(sysLogin.getLoginName());
        }

        return ResponseBean.success(sysUser);
    }

    /**
     * 查询用户默认密码
     */
    @RequestMapping(value="/admin/user/default/password",method= RequestMethod.GET)
    @ApiOperation(value="查询用户默认密码", notes = "查询用户默认密码", httpMethod = "GET", response = ResponseBean.class)
    @SaCheckPermission("user:view")
    public ResponseBean getDefaultPassword() throws Exception{
        return ResponseBean.success(defaultPassword);
    }
}
